// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "music.h"
#include "basic_dialogs.h"
#include "metadata_editor.h"
#include "album_selector.h"

#include <glibmm/i18n.h>

// Edit album metadata
void MusicView::EditAlbumMetadata()
{
	std::vector<Gtk::TreePath> selected_rows = get_selection()->get_selected_rows();
	Gtk::TreeIter album_iter = m_Store->get_iter(selected_rows.front());
	Glib::RefPtr<album_t> album =
		Glib::RefPtr<album_t>::cast_static(album_iter->get_value(m_StoreColumns.m_Data));

	AlbumMetadataEditor dialog(album);
	if (dialog.run() == Gtk::RESPONSE_OK &&
			((album->Id() && m_App.m_Device.UpdateAlbum(album)) ||
			 (!album->Id() && album->RawName().size() && m_App.m_Device.CreateNewAlbum(album))))
	{
		// Resort the model
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_UNSORTED_COLUMN_ID, Gtk::SORT_ASCENDING);
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_SORT_COLUMN_ID, Gtk::SORT_ASCENDING);

		// Scroll to the album
		scroll_to_cell(m_Store->get_path(album_iter), m_Column, 0.1f, 0.0f);
	}
}

// Add album art
void MusicView::AddAlbumArt()
{
	Glib::ustring image_filename = ImageChooserDialog(_("Select an image to use for album art"));
	if (image_filename.empty())
		return;

	std::vector<Gtk::TreePath> selected_rows = get_selection()->get_selected_rows();
	Gtk::TreeIter album_iter = m_Store->get_iter(selected_rows.front());
	Glib::RefPtr<album_t> album =
		Glib::RefPtr<album_t>::cast_static(album_iter->get_value(m_StoreColumns.m_Data));

	// If this is not an album on the device then prompt the user.
	if (!album->Id())
	{
		AlbumMetadataEditor dialog(album,
				_("This is not an actual album on the device.  "
					"If you want to create an album on the device enter a name for the album.  "
					"If you do not provide a name or cancel this dialog an album will not be "
					"created and album art will not be added."));
		if (dialog.run() != Gtk::RESPONSE_OK || album->RawName().empty()) return;
		if (!m_App.m_Device.CreateNewAlbum(album)) return;

		// Resort the model
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_UNSORTED_COLUMN_ID, Gtk::SORT_ASCENDING);
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_SORT_COLUMN_ID, Gtk::SORT_ASCENDING);

		// Scroll to the album
		scroll_to_cell(m_Store->get_path(album_iter), m_Column, 0.1f, 0.0f);
	}

	m_App.SetStatus(Glib::ustring::compose(_("Adding album art to %1"), album->Name()));
	if (m_App.m_Device.SetAlbumArt(album, image_filename))
	{
		// Update the album art for the tracks in the album
		for (Gtk::TreeIter track_iter = album_iter->children().begin();
				track_iter != album_iter->children().end(); ++track_iter)
		{
			Glib::RefPtr<track_t>::cast_static(track_iter->get_value(m_StoreColumns.m_Data))->art =
				album->art;
		}
	}
	m_App.UnsetStatus();
}

// Edit track metadata
void MusicView::EditTrackMetadata()
{
	std::vector<Gtk::TreePath> selected_rows = get_selection()->get_selected_rows();
	Gtk::TreeIter track_iter = m_Store->get_iter(selected_rows.front());
	Glib::RefPtr<track_t> track =
		Glib::RefPtr<track_t>::cast_static(track_iter->get_value(m_StoreColumns.m_Data));

	TrackMetadataEditor dialog(track);
	if (dialog.run() == Gtk::RESPONSE_OK && m_App.m_Device.UpdateTrack(track))
	{
		Gtk::TreePath orig_album_path = m_Store->get_path(track_iter);
		orig_album_path.up();
		Gtk::TreeIter orig_album_iter = m_Store->get_iter(orig_album_path);
		Glib::RefPtr<album_t> orig_album = 
			Glib::RefPtr<album_t>::cast_static(orig_album_iter->get_value(m_StoreColumns.m_Data));

		if (track->RawAlbum().collate_key() != orig_album->RawName().collate_key())
		{
			// Remove the track from the store
			m_Store->erase(track_iter);

			if (orig_album_iter->children().empty())
			{
				// The track's original album is empty so delete it
				if (orig_album->Id())
					m_App.m_Device.DeleteObject(orig_album->Id());

				m_Store->erase(orig_album_iter);
			}
			else
			{
				// Remove the track id from the old album's list
				std::vector<uint32_t> orig_album_tracks = orig_album->Tracks();
				std::vector<uint32_t>::iterator track_id_iter =
					std::find(orig_album_tracks.begin(), orig_album_tracks.end(), track->Id());
				if (track_id_iter != orig_album_tracks.end())
				{
					orig_album_tracks.erase(track_id_iter);
					orig_album->SetTracks(orig_album_tracks);
				}

				// Update the old album on the device
				if (orig_album->Id())
					m_App.m_Device.UpdateAlbum(orig_album);
			}

			// Add the track to its new album
			FindAlbumAndAddTrack(track);
		}

		// Resort the model
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_UNSORTED_COLUMN_ID, Gtk::SORT_ASCENDING);
		m_Store->set_sort_column(Gtk::TreeSortable::DEFAULT_SORT_COLUMN_ID, Gtk::SORT_ASCENDING);
	}
}

// Delete selected tracks from the device
void MusicView::DeleteTracksFromDevice()
{
	if (ConfirmationDialog(_("Delete Tracks"), _("Delete selected tracks from the device?")))
	{
		// Get the selected paths as row references
		std::vector<Gtk::TreeRowReference> selected_refs =
			ConvertPathsToRowRefs(get_selection()->get_selected_rows());

		for (std::vector<Gtk::TreeRowReference>::iterator ref = selected_refs.begin();
				ref != selected_refs.end(); ++ref)
		{
			Gtk::TreeIter iter = m_Store->get_iter(ref->get_path());

			if (Glib::RefPtr<track_t> track =
					Glib::RefPtr<track_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
			{
				// Remove the track from the device
				m_App.SetStatus(Glib::ustring::compose(_("Deleting track %1"), track->Name()));
				if (m_App.m_Device.DeleteObject(track->Id()))
				{
					// Store the album path
					Gtk::TreePath album_path(ref->get_path());
					album_path.up();

					// Remove the track from the store
					m_Store->erase(iter);

					// Remove all references to the track from the playlist view.
					m_App.PlaylistClearTrack(track->Id());

					// Get the track's album
					Gtk::TreeIter album_iter = m_Store->get_iter(album_path);
					Glib::RefPtr<album_t> album = 
						Glib::RefPtr<album_t>::cast_static(album_iter->get_value(m_StoreColumns.m_Data));

					// Remove the track from the album.  The track is
					// automatically removed from the album on the device, so
					// only playedit's internal reference needs to be updated.
					std::vector<uint32_t> album_tracks = album->Tracks();
					std::vector<uint32_t>::iterator track_id_iter =
						std::find(album_tracks.begin(), album_tracks.end(), track->Id());
					if (track_id_iter != album_tracks.end())
					{
						album_tracks.erase(track_id_iter);
						album->SetTracks(album_tracks);
					}

					// Check to see if the containing album is now empty, and if
					// so remove it from the device and store
					if (album_iter->children().empty())
					{
						m_App.SetStatus(Glib::ustring::compose(_("Deleting album %1"), album->Name()));
						m_App.m_Device.DeleteObject(album->Id());
						m_Store->erase(album_iter);
					}
				}
				m_App.UnsetStatus();
			}
		}
	}
}

// Delete selected albums from the device
void MusicView::DeleteAlbumsFromDevice()
{
	if (ConfirmationDialog(_("Delete Albums"), _("Delete selected albums from the device?")))
	{
		// Get the selected paths as row references
		std::vector<Gtk::TreeRowReference> selected_refs =
			ConvertPathsToRowRefs(get_selection()->get_selected_rows());

		for (std::vector<Gtk::TreeRowReference>::iterator ref = selected_refs.begin();
				ref != selected_refs.end(); ++ref)
		{
			Gtk::TreeIter iter = m_Store->get_iter(ref->get_path());

			if (Glib::RefPtr<album_t> album =
					Glib::RefPtr<album_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
			{
				std::vector<Gtk::TreeRowReference> track_refs = ConvertItersToRowRefs(iter->children());
				for (std::vector<Gtk::TreeRowReference>::iterator track_ref = track_refs.begin();
						track_ref != track_refs.end(); ++track_ref)
				{
					Gtk::TreeIter track_iter = m_Store->get_iter(track_ref->get_path());
					Glib::RefPtr<track_t> track =
						Glib::RefPtr<track_t>::cast_static(track_iter->get_value(m_StoreColumns.m_Data));
					m_App.SetStatus(Glib::ustring::compose(_("Deleting track %1"), track->Name()));
					if (track && m_App.m_Device.DeleteObject(track->Id()))
					{
						// Remove the track from the store
						m_Store->erase(track_iter);

						// Remove all references to the track from the playlist view.
						m_App.PlaylistClearTrack(track->Id());
					}
				}

				if (iter->children().empty())
				{
					m_App.SetStatus(Glib::ustring::compose(_("Deleting album %1"), album->Name()));
					m_App.m_Device.DeleteObject(album->Id());
					m_Store->erase(iter);
				}
				m_App.UnsetStatus();
			}
		}
	}
}

// Find an album for a track, prompting the user if more than one is found.
// If one is not found then create one.
void MusicView::FindAlbumAndAddTrack(Glib::RefPtr<track_t>& track)
{
	std::string track_album_key = track->Album().collate_key();

	// Find albums whose name match the album of the track
	std::vector<Gtk::TreeIter> matching_albums;
	for (Gtk::TreeIter album_iter = m_Store->children().begin();
			album_iter != m_Store->children().end(); ++album_iter)
	{
		if (Glib::RefPtr<album_t> album =
				Glib::RefPtr<album_t>::cast_dynamic(album_iter->get_value(m_StoreColumns.m_Data)))
		{
			if (album->Name().collate_key() == track_album_key)
				matching_albums.push_back(album_iter);
		}
	}

	Gtk::TreeIter album_iter;
	if (matching_albums.size() > 1)
	{
		int selected_album = AlbumSelector::Run(matching_albums, track);
		if (selected_album > -1)
			album_iter = matching_albums[selected_album];
	}
	else if (matching_albums.size() == 1)
		album_iter = matching_albums.front();

	Glib::RefPtr<album_t> album;
	if (!album_iter)
	{
		// Create the album and add it to the store
		album = album_t::create();
		album->SetName(track->RawAlbum());
		album->SetArtist(track->RawArtist());
		album->SetGenre(track->Genre());
		(*(album_iter = m_Store->append()))[m_StoreColumns.m_Data] = album;
	}
	else
		album = Glib::RefPtr<album_t>::cast_static(album_iter->get_value(m_StoreColumns.m_Data));

	std::vector<uint32_t> album_tracks = album->Tracks();
	album_tracks.push_back(track->Id());
	album->SetTracks(album_tracks);
	track->art = album->art;

	// Add the track to the store
	Gtk::TreeModel::Row track_row = *(m_Store->append(album_iter->children()));
	track_row[m_StoreColumns.m_Data] = track;

	// Create or update the album
	if (album->Id())
	{
		m_App.SetStatus(_("Updating album on device"));
		m_App.m_Device.UpdateAlbum(album);
	}
	else if (album->RawName().size())
	{
		m_App.SetStatus(_("Creating album on device"));
		m_App.m_Device.CreateNewAlbum(album);
	}

	// Scroll to the album and expand it
	scroll_to_cell(m_Store->get_path(album_iter), m_Column, 0.1f, 0.0f);
	expand_row(m_Store->get_path(album_iter), false);
}
