// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "treeview.h"
#include "tooltip.h"
#include <iostream>

TreeViewBase::TreeViewBase(const Glib::ustring& column_title)
	: m_Column(column_title)
{
	append_column(m_Column);

	m_Store = Gtk::TreeStore::create(m_StoreColumns);
	set_model(m_Store);

	// Set up the interactive search
	set_search_column(m_StoreColumns.m_Data);
	set_search_equal_func(sigc::mem_fun(*this, &TreeViewBase::SearchEqualFunc));

	// Set up tooltips
	property_has_tooltip() = true;
	signal_query_tooltip().connect(sigc::mem_fun(*this, &TreeViewBase::QueryTooltip));
}

// Toggles the expansion of a row in the treeview if it is double clicked.
void TreeViewBase::on_row_activated(const Gtk::TreePath& path, Gtk::TreeViewColumn* /* column */)
{
	if (row_expanded(path))
		collapse_row(path);
	else
		expand_row(path, false);
}

// Search equality function
bool TreeViewBase::SearchEqualFunc(const Glib::RefPtr<Gtk::TreeModel>& /* model */,
		int /* column */, const Glib::ustring& key, const Gtk::TreeIter& iter)
{
	Glib::RefPtr<mtp_data_t> mtp_data = (*iter)[m_StoreColumns.m_Data];

	return key.casefold_collate_key() != 
		mtp_data->Name().substr(0, key.length()).casefold_collate_key();
}

// Show a tooltip if the mouse hovers over a track, album, or playlist.
bool TreeViewBase::QueryTooltip(int x, int y, bool /* keyboard_tooltip */,
		const Glib::RefPtr<Gtk::Tooltip>& tooltip)
{
	Gtk::TreeIter iter;
	if (get_tooltip_context_iter(x, y, false, iter))
	{
		if (Glib::RefPtr<playlist_t> playlist =
				Glib::RefPtr<playlist_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
		{
			unsigned int duration = 0;
			for (Gtk::TreeIter child = iter->children().begin();
					child != iter->children().end(); ++child)
				duration += Glib::RefPtr<track_t>::cast_static(
						child->get_value(m_StoreColumns.m_Data))->Duration();
			tooltip->set_markup(GetPlaylistTooltipText(playlist, duration));
		}
		else if (Glib::RefPtr<track_t> track =
				Glib::RefPtr<track_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
		{
			tooltip->set_markup(GetTrackTooltipText(track));
			if (track->art)
				tooltip->set_icon(track->art);
		}
		else if (Glib::RefPtr<album_t> album =
				Glib::RefPtr<album_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
		{
			tooltip->set_markup(GetAlbumTooltipText(album));
			if (album->art)
				tooltip->set_icon(album->art);
		}
		else
			return false;

		set_tooltip_row(tooltip, m_Store->get_path(iter));
		return true;
	}
	else
		return false;
}

std::vector<Gtk::TreeRowReference> TreeViewBase::ConvertPathsToRowRefs(
		const std::vector<Gtk::TreePath>& paths)
{
	std::vector<Gtk::TreeRowReference> row_refs;
	for (std::vector<Gtk::TreePath>::const_iterator path = paths.begin();
			path != paths.end(); ++path)
		row_refs.push_back(Gtk::TreeRowReference(m_Store, *path));
	return row_refs;
}

std::vector<Gtk::TreeRowReference> TreeViewBase::ConvertItersToRowRefs(
		const Gtk::TreeNodeChildren& iters)
{
	std::vector<Gtk::TreeRowReference> row_refs;
	for (Gtk::TreeIter iter = iters.begin();
			iter != iters.end(); ++iter)
		row_refs.push_back(Gtk::TreeRowReference(m_Store, m_Store->get_path(iter)));
	return row_refs;
}

DNDTreeViewBase::DNDTreeViewBase(const Glib::ustring& column_title)
	: TreeViewBase(column_title),
	m_DragX(0), m_DragY(0)
{
}

// Reset drag data.
bool DNDTreeViewBase::on_button_release_event(GdkEventButton* event)
{
	Gtk::TreePath path;
	if ((m_DragX || m_DragY) &&
			!drag_check_threshold(m_DragX, m_DragY, event->x, event->y) &&
			get_path_at_pos(m_DragX, m_DragY, path))
	{
		set_cursor(path);
		Glib::RefPtr<Gtk::TreeSelection> selection = get_selection();
		if (selection->is_selected(path))
		{
			selection->unselect_all();
			selection->select(path);
		}
	}
	m_DragX = 0;
	m_DragY = 0;

	return Gtk::TreeView::on_button_release_event(event);
}

// Start a drag if needed.
bool DNDTreeViewBase::on_motion_notify_event(GdkEventMotion* event)
{
	if (m_DragX && m_DragY &&
			drag_check_threshold(m_DragX, m_DragY, event->x, event->y))
	{
		std::vector<Gtk::TargetEntry> target_entries;
		target_entries.push_back(Gtk::TargetEntry("GTK_TREE_MODEL_ROWS", Gtk::TARGET_SAME_APP, 0));
		Glib::RefPtr<Gtk::TargetList> target_list = Gtk::TargetList::create(target_entries);
		Glib::RefPtr<Gdk::DragContext> context = drag_begin(target_list,
				Gdk::ACTION_COPY | Gdk::ACTION_MOVE | Gdk::ACTION_LINK, 1, (GdkEvent*)event);
		return true;
	}

	return Gtk::TreeView::on_motion_notify_event(event);
}

void DNDTreeViewBase::on_drag_data_get(const Glib::RefPtr<Gdk::DragContext>& /* context */,
		Gtk::SelectionData& selection_data, guint /* info */, guint /* time */)
{
	selection_data.set(selection_data.get_target(), 0, NULL, 0);
}
